@@ -1,7 +1,6 @@
 //! Forest.
 
 mod builder;
-#[cfg(any(feature = "debug-print"))]
 mod debug_print;
 mod node;
 pub(crate) mod traverse;
@@ -10,12 +9,13 @@ use core::fmt;
 
 use alloc::vec::Vec;
 
-use crate::dynamic::hierarchy::traverse::{DftEvent, SafeModeDepthFirstTraverser};
+use crate::dynamic::hierarchy::traverse::{
+    DepthFirstTraverser, DftEvent, SafeModeDepthFirstTraverser,
+};
 use crate::dynamic::hierarchy::{Hierarchy, Neighbors};
-use crate::dynamic::{InsertAs, NodeId, NodeIdExt};
+use crate::dynamic::{InsertAs, InternalNodeId, NodeId, NodeIdExt};
 
 pub use self::builder::TreeBuilder;
-#[cfg(any(feature = "debug-print"))]
 pub use self::debug_print::DebugPrint;
 pub use self::node::{Node, NodeMut};
 
@@ -349,15 +349,17 @@ impl<Id: NodeId, T> Forest<Id, T> {
     ///
     /// # Panics
     ///
-    /// Panics if the node (the anchor of the destination) is not alive.
+    /// * Panics if the node (the anchor of the destination) is not alive.
+    /// * Panics if the node (the anchor of the destination) is not from this forest.
     #[inline]
     #[must_use = "newly created node cannot be accessed without the returned node ID"]
     pub fn create_insert(&mut self, data: T, dest: InsertAs<Id>) -> Id {
-        let new_id = self.create_root(data);
-        self.insert(new_id, dest)
-            .expect("[consistency] newly created node is independent from the destination");
-
-        new_id
+        Id::from_internal(create_insert_momo(
+            &mut self.hierarchy,
+            &mut self.data,
+            data,
+            dest.map(Id::to_internal),
+        ))
     }
 }
 
@@ -386,7 +388,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// some other node.
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// use treena::dynamic::InsertAs;
     ///
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
@@ -410,7 +411,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   |-- 1-1
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest.debug_print(root).to_string(), before);
     ///
     /// // Create a new node.
@@ -427,14 +427,12 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |-- new
     /// `-- 2"#;
     /// assert_eq!(forest.debug_print(root).to_string(), after_insert);
-    /// # }
     /// ```
     ///
     /// [`InsertAs::PreviousSiblingOf`] inserts the node as the previous sibling
     /// of some other node.
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// use treena::dynamic::InsertAs;
     ///
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
@@ -458,7 +456,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   |-- 1-1
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest.debug_print(root).to_string(), before);
     ///
     /// // Create a new node.
@@ -475,14 +472,12 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   `-- 1-2
     /// `-- 2"#;
     /// assert_eq!(forest.debug_print(root).to_string(), after_insert);
-    /// # }
     /// ```
     ///
     /// [`InsertAs::FirstChildOf`] inserts the node as the first child of some
     /// other node.
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// use treena::dynamic::InsertAs;
     ///
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
@@ -506,7 +501,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   |-- 1-1
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest.debug_print(root).to_string(), before);
     ///
     /// // Create a new node.
@@ -523,14 +517,12 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   `-- 1-2
     /// `-- 2"#;
     /// assert_eq!(forest.debug_print(root).to_string(), after_insert);
-    /// # }
     /// ```
     ///
     /// [`InsertAs::LastChildOf`] inserts the node as the last child of some
     /// other node.
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// use treena::dynamic::InsertAs;
     ///
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
@@ -554,7 +546,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   |-- 1-1
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest.debug_print(root).to_string(), before);
     ///
     /// // Create a new node.
@@ -571,7 +562,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   `-- new
     /// `-- 2"#;
     /// assert_eq!(forest.debug_print(root).to_string(), after_insert);
-    /// # }
     /// ```
     #[inline]
     pub fn insert(&mut self, node: Id, dest: InsertAs<Id>) -> Result<(), StructureError> {
@@ -597,7 +587,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// # Examples
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
     /// # let mut forest = Forest::<NodeIdUsize, _>::new();
     /// # let mut builder = TreeBuilder::new(&mut forest, "root");
@@ -619,7 +608,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   |-- 1-1
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest.debug_print(root).to_string(), before);
     ///
     /// // Detach the node "1".
@@ -634,7 +622,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// `-- 1-2"#;
     /// assert_eq!(forest.debug_print(root).to_string(), after_detach_root);
     /// assert_eq!(forest.debug_print(child_1).to_string(), after_detach_child_1);
-    /// # }
     /// ```
     #[inline]
     pub fn detach(&mut self, node: Id) {
@@ -660,7 +647,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// # Examples
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
     /// # let mut forest = Forest::<NodeIdUsize, _>::new();
     /// # let mut builder = TreeBuilder::new(&mut forest, "root");
@@ -682,7 +668,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   |-- 1-1
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest.debug_print(root).to_string(), before);
     ///
     /// // Detach the single node "1".
@@ -697,7 +682,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// let after_detach_child_1 = "1";
     /// assert_eq!(forest.debug_print(root).to_string(), after_detach_root);
     /// assert_eq!(forest.debug_print(child_1).to_string(), after_detach_child_1);
-    /// # }
     /// ```
     #[inline]
     pub fn detach_single(&mut self, node: Id) -> Result<(), StructureError> {
@@ -730,7 +714,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// # Examples
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
     /// # let mut forest = Forest::<NodeIdUsize, _>::new();
     /// # let mut builder = TreeBuilder::new(&mut forest, "root");
@@ -759,7 +742,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   |   `-- 1-1-2
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest.debug_print(root).to_string(), before);
     ///
     /// let mut removed_data = Vec::new();
@@ -777,44 +759,54 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// `-- 2"#;
     /// assert_eq!(forest.debug_print(root).to_string(), after_remove);
     /// assert_eq!(removed_data, &["1-1-0", "1-1-1", "1-1-2", "1-1"]);
-    /// # }
     /// ```
-    pub fn remove_hooked_panic_safe<F: FnMut(T)>(&mut self, node: Id, mut f: F) {
-        if !self.is_alive(node) {
-            return;
-        }
-        self.detach(node);
+    pub fn remove_hooked_panic_safe<F: FnMut(T)>(&mut self, node: Id, f: F) {
+        /// Implementation that depends on `Id::Internal` instead of `NodeId`
+        /// in order to reduce monomorphization and prevent binary bloat.
+        fn inner<InternalId: InternalNodeId, T, F: FnMut(T)>(
+            hierarchy: &mut Hierarchy<InternalId>,
+            data: &mut Vec<Option<T>>,
+            node: InternalId,
+            mut f: F,
+        ) {
+            if !hierarchy.is_alive(node) {
+                return;
+            }
+            hierarchy.detach(node);
 
-        let mut traverser = SafeModeDepthFirstTraverser::new(node.to_internal(), &self.hierarchy);
-        while let Some(ev) = traverser.next(&self.hierarchy) {
-            let id = match ev {
-                DftEvent::Open(_) => continue,
-                DftEvent::Close(id) => Id::from_internal(id),
-            };
+            let mut traverser = SafeModeDepthFirstTraverser::new(node, hierarchy);
+            while let Some(ev) = traverser.next(hierarchy) {
+                let id = match ev {
+                    DftEvent::Open(_) => continue,
+                    DftEvent::Close(id) => id,
+                };
 
-            // Detach the leaf node.
-            debug_assert!(
-                self.neighbors(id)
-                    .expect("[consistency] the current node must be alive here")
-                    .first_child()
-                    .is_none(),
-                "[consistency] the node must be the leaf"
-            );
-            self.detach(id);
-            let nbs = self
-                .hierarchy
-                .neighbors_mut(id.to_internal())
-                .expect("[consistency] the current node must be alive here");
-            debug_assert!(
-                nbs.is_alone(),
-                "[consistency] the detached leaf node must be alone"
-            );
-            nbs.make_removed();
-            let data = self.data[id.to_usize()]
-                .take()
-                .expect("[consistency] the node must have an associated data");
-            f(data);
+                // Detach the leaf node.
+                debug_assert!(
+                    hierarchy
+                        .neighbors(id)
+                        .expect("[consistency] the current node must be alive here")
+                        .first_child()
+                        .is_none(),
+                    "[consistency] the node must be the leaf"
+                );
+                hierarchy.detach(id);
+                let nbs = hierarchy
+                    .neighbors_mut(id)
+                    .expect("[consistency] the current node must be alive here");
+                debug_assert!(
+                    nbs.is_alone(),
+                    "[consistency] the detached leaf node must be alone"
+                );
+                nbs.make_removed();
+                let elem = data[id.to_usize()]
+                    .take()
+                    .expect("[consistency] the node must have an associated data");
+                f(elem);
+            }
         }
+
+        inner(&mut self.hierarchy, &mut self.data, node.to_internal(), f)
     }
 
     /// Removes the subtree from the forest.
@@ -837,7 +829,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// # Examples
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
     /// # let mut forest = Forest::<NodeIdUsize, _>::new();
     /// # let mut builder = TreeBuilder::new(&mut forest, "root");
@@ -866,7 +857,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   |   `-- 1-1-2
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest.debug_print(root).to_string(), before);
     ///
     /// let mut removed_data = Vec::new();
@@ -884,34 +874,42 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// `-- 2"#;
     /// assert_eq!(forest.debug_print(root).to_string(), after_remove);
     /// assert_eq!(removed_data, &["1-1-0", "1-1-1", "1-1-2", "1-1"]);
-    /// # }
     /// ```
-    pub fn remove_hooked<F: FnMut(T)>(&mut self, node: Id, mut f: F) {
-        if !self.is_alive(node) {
-            return;
-        }
-        self.detach(node);
+    pub fn remove_hooked<F: FnMut(T)>(&mut self, node: Id, f: F) {
+        /// Implementation that depends on `Id::Internal` instead of `NodeId`
+        /// in order to reduce monomorphization and prevent binary bloat.
+        fn inner<InternalId: InternalNodeId, T, F: FnMut(T)>(
+            hierarchy: &mut Hierarchy<InternalId>,
+            data: &mut Vec<Option<T>>,
+            node: InternalId,
+            mut f: F,
+        ) {
+            if !hierarchy.is_alive(node) {
+                return;
+            }
+            hierarchy.detach(node);
 
-        let mut traverser = SafeModeDepthFirstTraverser::new(node.to_internal(), &self.hierarchy);
-        while let Some(ev) = traverser.next(&self.hierarchy) {
-            let id = match ev {
-                DftEvent::Open(_) => continue,
-                DftEvent::Close(id) => id,
-            };
+            let mut traverser = SafeModeDepthFirstTraverser::new(node, hierarchy);
+            while let Some(ev) = traverser.next(hierarchy) {
+                let id = match ev {
+                    DftEvent::Open(_) => continue,
+                    DftEvent::Close(id) => id,
+                };
 
-            // Break the node. The tree will be temporarily inconsistent.
-            let nbs = self
-                .hierarchy
-                .neighbors_mut(id)
-                .expect("[consistency] the current node must be alive here");
-            nbs.force_make_removed();
-            let data = self.data[id.to_usize()]
-                .take()
-                .expect("[consistency] the node must have an associated data");
-            f(data);
+                // Break the node. The tree will be temporarily inconsistent.
+                let nbs = hierarchy
+                    .neighbors_mut(id)
+                    .expect("[consistency] the current node must be alive here");
+                nbs.force_make_removed();
+                let data = data[id.to_usize()]
+                    .take()
+                    .expect("[consistency] the node must have an associated data");
+                f(data);
+            }
+            // At this point, all nodes under the `node` are removed.
+            // Now the forest must be totally consistent.
         }
-        // At this point, all nodes under the `node` are removed.
-        // Now the forest must be totally consistent.
+        inner(&mut self.hierarchy, &mut self.data, node.to_internal(), f);
     }
 
     /// Removes the subtree from the forest.
@@ -929,7 +927,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// # Examples
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
     /// # let mut forest = Forest::<NodeIdUsize, _>::new();
     /// # let mut builder = TreeBuilder::new(&mut forest, "root");
@@ -958,7 +955,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   |   `-- 1-1-2
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest.debug_print(root).to_string(), before);
     ///
     /// // Remove "1-1" and its descendant.
@@ -973,7 +969,6 @@ impl<Id: NodeId, T> Forest<Id, T> {
     /// |   `-- 1-2
     /// `-- 2"#;
     /// assert_eq!(forest.debug_print(root).to_string(), after_remove);
-    /// # }
     /// ```
     #[inline]
     pub fn remove(&mut self, node: Id) {
@@ -1512,7 +1507,6 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     /// # Examples
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
     /// # let mut forest = Forest::<NodeIdUsize, _>::new();
     /// # let mut builder = TreeBuilder::new(&mut forest, "root");
@@ -1534,7 +1528,6 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     /// |   |-- 1-1
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest.debug_print(root).to_string(), before);
     ///
     /// // Clone a tree.
@@ -1549,7 +1542,6 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     ///     forest.node(cloned).expect("must be alive").parent_id().is_none(),
     ///     "The new node is a root node of an independent tree and has no parent"
     /// );
-    /// # }
     /// ```
     #[must_use = "newly created node cannot be accessed without the returned node ID"]
     pub fn clone_local_tree(&mut self, src_id: Id) -> Id {
@@ -1566,7 +1558,6 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     /// # Examples
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
     /// # let mut forest_src = Forest::<NodeIdUsize, _>::new();
     /// # let mut builder = TreeBuilder::new(&mut forest_src, "root");
@@ -1588,7 +1579,6 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     /// |   |-- 1-1
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest_src.debug_print(root).to_string(), before);
     ///
     /// // Destination forest.
@@ -1607,7 +1597,6 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     ///     forest_dest.node(cloned).expect("must be alive").parent_id().is_none(),
     ///     "The new node is a root node of an independent tree and has no parent"
     /// );
-    /// # }
     /// ```
     #[must_use = "newly created node cannot be accessed without the returned node ID"]
     pub fn clone_foreign_tree(&mut self, src_root: Node<'_, Id, T>) -> Id {
@@ -1623,7 +1612,6 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     /// # Examples
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
     /// # let mut forest = Forest::<NodeIdUsize, _>::new();
     /// # let mut builder = TreeBuilder::new(&mut forest, "root");
@@ -1645,7 +1633,6 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     /// |   |-- 1-1
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest.debug_print(root).to_string(), before);
     ///
     /// // Clone a tree.
@@ -1669,56 +1656,80 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     /// for (src, dest) in mapping {
     ///     assert_eq!(forest.data(src), forest.data(dest));
     /// }
-    /// # }
     /// ```
     #[must_use = "newly created node cannot be accessed without the returned node ID"]
     pub fn clone_local_tree_with_id_mapping<F>(&mut self, src_id: Id, mut add_mapping: F) -> Id
     where
         F: FnMut(Id, Id),
     {
+        /// Clones the next node.
+        ///
+        /// Returns `Option<SourceId>`.
+        ///
+        /// Implementation that depends on `Id::Internal` instead of `NodeId`
+        /// in order to reduce monomorphization and prevent binary bloat.
+        fn clone_next_node<InternalId: InternalNodeId, T: Clone>(
+            hierarchy: &mut Hierarchy<InternalId>,
+            data: &mut Vec<Option<T>>,
+            current_dest: &mut InternalId,
+            traverser: &mut SafeModeDepthFirstTraverser<InternalId>,
+        ) -> Option<InternalId> {
+            while let Some(ev) = traverser.next(hierarchy) {
+                match ev {
+                    DftEvent::Open(src_id) => {
+                        let node_data = data
+                            .get(src_id.to_usize())
+                            .cloned()
+                            .expect("[consistency] the node being traversed must be alive")
+                            .expect("[consistency] the node being traversed must be alive");
+                        *current_dest = create_insert_momo(
+                            hierarchy,
+                            data,
+                            node_data,
+                            InsertAs::LastChildOf(*current_dest),
+                        );
+                        return Some(src_id);
+                    }
+                    DftEvent::Close(_) => {
+                        let parent = hierarchy
+                            .neighbors(*current_dest)
+                            .expect("[consistency] the node being created must be alive")
+                            .parent();
+                        *current_dest = match parent {
+                            Some(id) => id,
+                            None => return None,
+                        }
+                    }
+                }
+            }
+            unreachable!("[consistency] traversal must end with the closing of the root node");
+        }
+
         let root_data = self
             .data(src_id)
             .expect("[consistency] the node must be alive")
             .clone();
         let root_id = self.create_root(root_data);
-        let mut current_dest = root_id;
-        add_mapping(src_id, current_dest);
+        add_mapping(src_id, root_id);
         let mut traverser = SafeModeDepthFirstTraverser::new(src_id.to_internal(), &self.hierarchy);
 
         // Skip the open event of the root node.
         let _ = traverser.next(&self.hierarchy);
 
-        while let Some(ev) = traverser.next(&self.hierarchy) {
-            match ev {
-                DftEvent::Open(src_id) => {
-                    let src_id = Id::from_internal(src_id);
-                    let data = self
-                        .data(src_id)
-                        .expect("[consistency] the node being traversed must be alive")
-                        .clone();
-                    current_dest = self.create_insert(data, InsertAs::LastChildOf(current_dest));
-                    add_mapping(src_id, current_dest);
-                }
-                DftEvent::Close(_) => {
-                    let parent = self
-                        .node(current_dest)
-                        .expect("[consistency] the node being created must be alive")
-                        .parent_id();
-                    current_dest = match parent {
-                        Some(id) => id,
-                        None => {
-                            assert_eq!(
-                                current_dest, root_id,
-                                "[consistency] current node must be the root if it has no parent"
-                            );
-                            break;
-                        }
-                    }
-                }
-            }
+        // Clone descendants.
+        let mut current_dest = root_id.to_internal();
+        while let Some(src_id) = clone_next_node(
+            &mut self.hierarchy,
+            &mut self.data,
+            &mut current_dest,
+            &mut traverser,
+        ) {
+            add_mapping(Id::from_internal(src_id), Id::from_internal(current_dest));
         }
+
         assert_eq!(
-            current_dest, root_id,
+            Id::from_internal(current_dest),
+            root_id,
             "[consistency] all nodes including the root must have been closed"
         );
 
@@ -1735,7 +1746,6 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     /// # Examples
     ///
     /// ```
-    /// # #[cfg(feature = "debug-print")] {
     /// # use treena::dynamic::{Forest, NodeIdUsize, TreeBuilder};
     /// # let mut forest_src = Forest::<NodeIdUsize, _>::new();
     /// # let mut builder = TreeBuilder::new(&mut forest_src, "root");
@@ -1757,7 +1767,6 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     /// |   |-- 1-1
     /// |   `-- 1-2
     /// `-- 2"#;
-    /// // NOTE: `.debug_print()` requires `debug-print` feature to be enabled.
     /// assert_eq!(forest_src.debug_print(root).to_string(), before);
     ///
     /// // Destination forest.
@@ -1785,7 +1794,6 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     /// for (src, dest) in mapping {
     ///     assert_eq!(forest_src.data(src), forest_dest.data(dest));
     /// }
-    /// # }
     /// ```
     #[must_use = "newly created node cannot be accessed without the returned node ID"]
     pub fn clone_foreign_tree_with_id_mapping<F>(
@@ -1796,51 +1804,98 @@ impl<Id: NodeId, T: Clone> Forest<Id, T> {
     where
         F: FnMut(Id, Id),
     {
-        use crate::dynamic::forest::traverse::DftEvent;
-
-        let mut events = src_root.depth_first_traverse();
-        let root_data = match events.next() {
-            None => unreachable!(
-                "[validity] iterator must emit at least two events since it has the root node"
-            ),
-            Some(DftEvent::Open(node)) => node.data().clone(),
-            Some(DftEvent::Close(_)) => {
-                unreachable!("[validity] `DepthFirstTraverse` must emit the open event first")
-            }
-        };
-        let mut builder = TreeBuilder::new(self, root_data);
-        let root_id = builder.root_id();
-        add_mapping(src_root.id(), root_id);
-
-        for ev in events {
-            match ev {
-                DftEvent::Open(node) => {
-                    let data = node.data().clone();
-                    builder.child(data);
-                    add_mapping(node.id(), builder.current_id());
-                }
-                DftEvent::Close(_) => {
-                    // When `builder.try_parent().is_none()` is true, it
-                    // means that the node being closed is the root node of the
-                    // new tree. It is not an error and no action needed here,
-                    // since no more events will be provided from `events`.
-                    let _ = builder.try_parent();
+        /// Clones the next node.
+        ///
+        /// Returns `Option<SourceId>`.
+        ///
+        /// Implementation that depends on `Id::Internal` instead of `NodeId`
+        /// in order to reduce monomorphization and prevent binary bloat.
+        fn clone_next_node<InternalId: InternalNodeId, T: Clone>(
+            hierarchy_dest: &mut Hierarchy<InternalId>,
+            data_dest: &mut Vec<Option<T>>,
+            current_dest: &mut InternalId,
+            hierarchy_src: &Hierarchy<InternalId>,
+            src_traverser: &mut DepthFirstTraverser<InternalId>,
+            data_src: &[Option<T>],
+        ) -> Option<InternalId> {
+            while let Some(ev) = src_traverser.next(hierarchy_src) {
+                match ev {
+                    DftEvent::Open(src_id) => {
+                        let node_data = data_src
+                            .get(src_id.to_usize())
+                            .cloned()
+                            .expect("[consistency] the node being traversed must be alive")
+                            .expect("[consistency] the node being traversed must be alive");
+                        *current_dest = create_insert_momo(
+                            hierarchy_dest,
+                            data_dest,
+                            node_data,
+                            InsertAs::LastChildOf(*current_dest),
+                        );
+                        return Some(src_id);
+                    }
+                    DftEvent::Close(_) => {
+                        match hierarchy_dest
+                            .neighbors(*current_dest)
+                            .expect("[consistency] the node must be created recently and alive")
+                            .parent()
+                        {
+                            Some(parent) => {
+                                *current_dest = parent;
+                                continue;
+                            }
+                            None => {
+                                // Closing root node.
+                                return None;
+                            }
+                        }
+                    }
                 }
             }
+            unreachable!("[consistency] traversal must end with the closing of the root node");
         }
 
-        root_id
+        let root_data = src_root.data().clone();
+        let root_dest_id = self.create_root(root_data);
+        let hierarchy_src = &src_root.forest().hierarchy;
+        add_mapping(src_root.id(), root_dest_id);
+        let mut src_traverser =
+            DepthFirstTraverser::with_toplevel(src_root.id().to_internal(), hierarchy_src);
+
+        // Skip the open event of the root node.
+        let _ = src_traverser.next(hierarchy_src);
+
+        // Clone descendants.
+        let mut current_dest = root_dest_id.to_internal();
+        while let Some(src_id) = clone_next_node(
+            &mut self.hierarchy,
+            &mut self.data,
+            &mut current_dest,
+            hierarchy_src,
+            &mut src_traverser,
+            &src_root.forest().data,
+        ) {
+            add_mapping(Id::from_internal(src_id), Id::from_internal(current_dest));
+        }
+
+        assert_eq!(
+            Id::from_internal(current_dest),
+            root_dest_id,
+            "[consistency] traversal must end with the closing of the destination root node"
+        );
+        assert_eq!(
+            src_traverser.next(hierarchy_src),
+            None,
+            "[consistency] traversal must also end with the closing of the source root node"
+        );
+
+        root_dest_id
     }
 }
 
 /// Debug printing.
-#[cfg(feature = "debug-print")]
 impl<Id: NodeId, T> Forest<Id, T> {
     /// Returns the pretty-printable proxy object to the node and descendants.
-    ///
-    /// This requires `debug-print` feature to be enabled.
-    #[cfg(feature = "debug-print")]
-    #[cfg_attr(feature = "docsrs", doc(cfg(feature = "debug-print")))]
     pub fn debug_print(&self, id: Id) -> debug_print::DebugPrint<'_, Id, T> {
         let node = self
             .node(id)
@@ -1858,6 +1913,39 @@ impl<Id: NodeId, T> Default for Forest<Id, T> {
     }
 }
 
+/// Creates a node and inserts it to the target position.
+///
+/// For the spec and limitations, see [`Forest::create_insert`].
+///
+/// Implementation that depends on `Id::Internal` instead of `NodeId`
+/// in order to reduce monomorphization and prevent binary bloat.
+// To avoid duplicate document that is hard to maintain, specs are described
+// in the doc comments for `Forest::create_insert`.
+fn create_insert_momo<Id: InternalNodeId, T>(
+    hierarchy: &mut Hierarchy<Id>,
+    storage: &mut Vec<Option<T>>,
+    data: T,
+    dest: InsertAs<Id>,
+) -> Id {
+    assert!(
+        hierarchy.is_valid(dest.to_anchor()),
+        "[precondition] the anchor must be valid for this forest"
+    );
+    let new_id = hierarchy.create_root();
+    storage.push(Some(data));
+    // This insert must not fail if the anchor of `dest` is alive and from the forest.
+    // If the anchor is not alive, `insert()` itself panics.
+    //
+    // If the anchor is from other forest but accidentally valid as the node ID
+    // as this forest, `insert()` could return `Err(_)`.
+    hierarchy.insert(new_id, dest).expect(
+        "[precondition] newly created node is independent from the destination \
+         and the anchor has been valid before the new node is created",
+    );
+
+    new_id
+}
+
 /// Structure inconsistency error.
 #[derive(Debug, Clone, Copy)]
 pub enum StructureError {
@@ -1882,3 +1970,87 @@ impl fmt::Display for StructureError {
 
 #[cfg(feature = "std")]
 impl std::error::Error for StructureError {}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    /// Ensures that functions defined in functions are monomorphized
+    /// independently without considering outer function's type parameters.
+    ///
+    /// This is in fact testing compiler's behavior, rather than the
+    /// implementation of this crate.
+    ///
+    /// This fact is useful to reduce monomorphization.
+    ///
+    /// `Forest<Id, T>` is a pair of `Hierarchy<Id::Internal>` and `Vec<T>`, and
+    /// most of the operations provided from `Forest<Id, T>` actually depends on
+    /// `T` and `Id::Internal`, not `Id`.
+    /// `Id` is only necessary to make parameter types and return types
+    /// different, and the internal logic is almost identical when
+    /// `Id::Internal` is the same.
+    ///
+    /// For example, internal implementantion of `Forest::<Id0, T>::remove` and
+    /// `Forest::<Id1, T>::remove` can be almost the same when `Id0::Internal`
+    /// and `Id1::Internal` are the same type, except the part that converts
+    /// `Id0` or `Id1` into `_::Internal` type value.
+    #[test]
+    fn ensure_inner_function_is_monomorphized() {
+        use core::any::Any;
+
+        use crate::dynamic::{InternalNodeId, NodeIdUsize};
+        use crate::impl_dynamic_node_id;
+
+        // Dummy struct mimicking `Forest<Id: NdoeId, T>`.
+        struct MyContainer<Id: NodeId, T> {
+            _ids: Vec<Id::Internal>,
+            _values: Vec<T>,
+        }
+        impl<Id: NodeId, T> Default for MyContainer<Id, T> {
+            fn default() -> Self {
+                Self {
+                    _ids: Default::default(),
+                    _values: Default::default(),
+                }
+            }
+        }
+
+        impl<Id: NodeId, T> MyContainer<Id, T> {
+            fn new() -> Self {
+                Self::default()
+            }
+
+            fn inner_fn_addr(&self) -> fn(Id::Internal) -> () {
+                fn inner<I: InternalNodeId>(_: I) {}
+
+                inner::<Id::Internal> as fn(Id::Internal) -> ()
+            }
+        }
+
+        #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+        struct MyId0(NodeIdUsize);
+        impl_dynamic_node_id!(MyId0, NodeIdUsize, 0);
+
+        #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+        struct MyId1(NodeIdUsize);
+        impl_dynamic_node_id!(MyId1, NodeIdUsize, 0);
+
+        let container0 = MyContainer::<MyId0, usize>::new();
+        let container1 = MyContainer::<MyId1, usize>::new();
+
+        let outer_addr0 = MyContainer::<MyId0, usize>::inner_fn_addr as fn(_) -> _;
+        let outer_addr1 = MyContainer::<MyId1, usize>::inner_fn_addr as fn(_) -> _;
+
+        assert_ne!(
+            outer_addr0.type_id(),
+            outer_addr1.type_id(),
+            "outer functions must have different types"
+        );
+        assert_eq!(
+            container0.inner_fn_addr(),
+            container1.inner_fn_addr(),
+            "inner functions must have the same address \
+             while the outer functions are different"
+        );
+    }
+}
